@@ -7,8 +7,8 @@
 ### vcov, logLik, print, summary, plot (intensity, iaf), R0, residuals, update
 ###
 ### Copyright (C) 2009-2014 Sebastian Meyer
-### $Revision: 988 $
-### $Date: 2014-09-01 16:58:44 +0200 (Mon, 01 Sep 2014) $
+### $Revision: 1147 $
+### $Date: 2014-12-19 11:20:48 +0100 (Fri, 19 Dec 2014) $
 ################################################################################
 
 
@@ -263,9 +263,9 @@ ret
 
 xtable.summary.twinstim <- function (x, caption = NULL, label = NULL,
                              align = c("l", "r", "r", "r"), digits = 3,
-                             display = c("s", "f", "s", "s"),
+                             display = c("s", "f", "s", "s"), ...,
                              ci.level = 0.95, ci.fmt = "%4.2f", ci.to = "--",
-                             eps.Pvalue = 1e-4, ...)
+                             eps.Pvalue = 1e-4)
 {
     cis <- confint(x, level=ci.level)
     tabh <- x$coefficients.beta
